#include <sta/spi/settings.hpp>

#include <sta/assert.hpp>
#include <sta/lang.hpp>


namespace sta
{
    SPIClkPolarity getSPIClkPolarity(SPIMode mode)
    {
        switch (mode)
        {
        case SPIMode::MODE_0:
        case SPIMode::MODE_1:
            return SPIClkPolarity::LOW;

        case SPIMode::MODE_2:
        case SPIMode::MODE_3:
            return SPIClkPolarity::HIGH;

        default:
            // Unreachable case
            STA_ASSERT_MSG(false, "Case for SPIMode enum not handled");
            STA_UNREACHABLE();
        }
    }

    SPIClkPhase getSPIClkPhase(SPIMode mode)
    {
        switch (mode)
        {
        case SPIMode::MODE_0:
        case SPIMode::MODE_2:
            return SPIClkPhase::EDGE_1;

        case SPIMode::MODE_1:
        case SPIMode::MODE_3:
            return SPIClkPhase::EDGE_2;

        default:
            // Unreachable case
            STA_ASSERT_MSG(false, "Case for SPIMode enum not handled");
            STA_UNREACHABLE();
        }
    }

    SPIMode getSPIMode(SPIClkPolarity polarity, SPIClkPhase phase)
    {
        if (polarity == SPIClkPolarity::LOW)
        {
            if (phase == SPIClkPhase::EDGE_1)
            {
                return SPIMode::MODE_0;
            }
            else
            {
                return SPIMode::MODE_1;
            }
        }
        else
        {
            if (phase == SPIClkPhase::EDGE_1)
            {
                return SPIMode::MODE_2;
            }
            else
            {
                return SPIMode::MODE_3;
            }
        }
    }
} // namespace sta
